import struct

def write_ld(writer, protomsg):
    """
    Write a length delimited protobuf message to the provided writer
    
    Arguments:
        writer: a writer, e.g. a file opened in 'wb' mode or a BytesIO or StringIO object
        protomsg: a protobuf message
    """
    
    # serialize the message to a bytes array
    s = protomsg.SerializeToString()
    
    # get the length of the message (unsigned 32-bit integer) as a bytes array of size 4
    len_uint64_bytes = struct.pack('>L', len(s))
    
    # return the length + serialized string
    writer.write(len_uint64_bytes + s)


def read_ld(reader, msgtype):
    """
    Reads a length-delimited protobuf message from the provided reader.
    
    Arguments:
        reader: a reader, e.g. a file opened with 'rb' or a BytesIO or StringIO object
        msgtype: the descriptor of the protobuf message, typically the name of the message,
            e.g. mypb.User
    """
    assert reader is not None, "reader is required"
    assert msgtype is not None, "msgtype is required"
    
    # read 4 bytes from the message
    msg_len_bytes = reader.read(4)
    
    # EOF yields 0 bytes when reading 
    if len(msg_len_bytes) == 0:
        return None
    
    # parse the 8 bytes as a 32-bit unsigned long
    msg_len_tuple = struct.unpack('>L', msg_len_bytes)
    
    # struct.unpack always returns a tuple, in this case with one entry - the message length
    msg_len = msg_len_tuple[0]

    # read the message as a byte string
    proto_str = reader.read(msg_len)
    
    # EOF yields 0 bytes when reading 
    if len(msg_len_bytes) == 0:
        return None
    
    # de-serialize the bytes string as the msgtype
    msg = msgtype()
    
    # return the protobuf message
    return msg.ParseFromString(proto_str)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ldproto']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'ldproto',
    'version': '0.1.1',
    'description': '',
    'long_description': '# Length-Delimited Proto\n\nWhen protobuf messages are either across the wire, or put in intermediary storage, it is helpful to be able to read and write individual messages in a streaming format.\n\nThis package exposes two methods:\n\n* `write_ld(writer, protomsg)` - writes one instance of a protobuf message to the stream\n* `read_ld(reader, msgtype) -> protomsg` - reads one protobuf message from the stream, using the type as the constructor\n\nThis package uses an unsigned 32-bit integer as the length-prefix.\n\n## Example\n\nAssuming there is a protobuf message with the type name "User"\n\n```python\nfrom ldproto import read_ld, write_ld\nimport myproto as pb\n\n# .ld is for length-delimited\nwith open(\'out.user.ld\', \'wb\') as f:\n    for user in users:\n        write_ld(f, user)\n\nparsed_users = []\nwith open(\'out.user.ld\', \'rb\') as f:\n    while True:\n        # Replace pb.User here with the protobuf\n        user = read_ld(f, pb.User)\n        if user is None:\n            print(\'done reading messages\')\n            break\n        parsed_users.append(user)\n```\n\nTo write to / from a bytestream in-memory, use BytesIO in-place of the files in the example.\n',
    'author': 'Sebastian Nyberg',
    'author_email': 'seb.nyberg90@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
